`timescale 1ns / 1ns

module fir_test;

reg clk_pll, reset;

wire [7:0] nco_out;

reg [15:0]	ncoA_freq0_r1, ncoA_freq0_r0,
			ncoA_waveform, ncoA_ph_offset,
			chanA_gain, offset;

NCO dds
(
	.clk_i(clk_pll),
    .out1(nco_out)
);
  reg           [13:0]	dac1_d;
  reg 	signed	[7:0] signal1;	
  wire 	signed	[18:0] signal2;	
  wire 	signed	[11:0] signal3;
  wire 	signed	[11:0] signal4;
  
  wire dw;

  // Данные с АЦП переводим в знаковую форму: 
  // assign signal1 = nco_out;



  // mult test
  // (
    // .h(9'd255),
    // .s(8'd127),
    // .m(signal2)
  // );

  // Экземпляр модуля фильтра
  fir 
  #(
    .N(13),    // количество коэффициентов фильтра
    .H_W(9),  // разрядность коэффициентов
    .DI_W(8), // разрядность входных данных
    .DO_W(19) // разрядность выходных данных
    ) fir_inst
  (
    .clk_i (clk_pll),
    .data_i (signal1),
    .data_o (signal2)
   );



 	always @(posedge clk_pll)
	begin
		// Масштабируем  сигнал и переводим его в беззнаковую форму
    dac1_d <= (signal4 << 2) + $signed({1'b1, {(13){1'b0}}});	
			
	end

//моделируем сигнал тактовой частоты
always
  #5 clk_pll = ~clk_pll;
 

//заканчиваем симуляцию в момент времени "400"
initial
begin

    reset = 1;
    clk_pll = 0;
    
    #10  reset = 0;
    #5 signal1 = 0;
    
    
    #150 signal1 = 127;
    #400 signal1 = 0;
    
    
    
    
    #4000 $finish;
end

//создаем файл VCD для последующего анализа сигналов
initial
begin
  $dumpfile("out.vcd");
  $dumpvars(2,fir_test);
  end

//наблюдаем на некоторыми сигналами системы
//initial
//$monitor($stime,, enable,, clk,,tick);

endmodule


module mult
(
  input wire signed [8:0] h,
  input wire signed [7:0] s,
  output wire signed [16:0] m
);
  assign m = h * s;

endmodule


(* multstyle = "dsp" *) module  fir
#(
  parameter     N = 5,      // количество коэффициентов 
  parameter     H_W = 9,    // разрядность h 
  parameter     DI_W = 8,   // разрядность входных данных
  parameter     DO_W = 18   // разрядность выходных данных
)
(  
  input wire                    clk_i,
  input wire  signed [DI_W-1:0] data_i,
  output  reg        [DO_W-1:0] data_o 
);
// Линию задержки реализуем в виде массива регистров,
// разрядность слова DI_W, число элементов N-1 -- элементов задержки
// должно быть на единицу меньше, чем число отсчетов в h.   
  reg  signed [DI_W-1:0] r [N-2:0];   // N-1 регистров                     

  // Коэффициенты фильтра
  reg  signed [H_W-1:0] h [N-1:0];   // N регистров  

  // Результат умножения
  reg  signed [H_W + DI_W -1:0] m [N-1:0];
   
  initial begin
    // h[0] = -53;
    // h[1] = 138;
    // h[2] = 255;
    // h[3] = 138; 
    // h[4] = -53;
    // // Полосовой
    // h[0] = -23;
    // h[1] = -51;
    // h[2] = 128;
    // h[3] = 68; 
    // h[4] = -219;
    // h[5] = -30;
    // h[6] = 255;
    // h[7] = -30;
    // h[8] = -219; 
    // h[9] = 68;
    // h[10] = 128;
    // h[11] = -51;
    // h[12] = -23;
    
    // h[0] = -80;
    // h[1] = -42;
    // h[2] = 25;
    // h[3] = 105; 
    // h[4] = 182;
    // h[5] = 236;
    // h[6] = 255;
    // h[7] = 236;
    // h[8] = 182; 
    // h[9] = 105;
    // h[10] = 25;
    // h[11] = -42;
    // h[12] = -80;
    // // ФНЧ: N=13, 8кгЦ, 15кГц.
    // h[0] = -74;
    // h[1] = -39;
    // h[2] = 23;
    // h[3] = 98; 
    // h[4] = 168;
    // h[5] = 218;
    // h[6] = 236;
    // h[7] = 218;
    // h[8] = 168; 
    // h[9] = 98;
    // h[10] = 23;
    // h[11] = -39;
    // h[12] = -74;
    
    // ФНЧ: N=13, 16кгЦ, 23кГц.
    h[0] = 42;
    h[1] = -13;
    h[2] = -62;
    h[3] = -36; 
    h[4] = 73;
    h[5] = 199;
    h[6] = 255;
    h[7] = 199;
    h[8] = 73; 
    h[9] = -36;
    h[10] = -62;
    h[11] = -13;
    h[12] = 42;
    
  end

  integer i;
  reg  signed    [DO_W-1:0]    acc;

  always @(posedge clk_i) begin
    for(i = 0; i < N-1; i = i + 1)
      if (i == 0)
        r[i] <= data_i;
      else
        r[i] <= r[i-1];
  end
  
  always @(*) begin
    for(i = 0; i < N; i = i + 1)  
      if (i == 0)
        m[i] <= data_i * h[i];
      else
        m[i] <= r[i-1] * h[i];
  end
 
  always @(*) begin
    for(i = 0; i < N; i = i + 1)
      if (i == 0)
        acc = m[i];
      else
        acc = acc + m[i];
  end
  

always @(posedge clk_i) 
    data_o = acc; 
    
endmodule



module NCO(

	input				clk_i,	// Вход с тактового генератора
	
	output	reg [7:0]	out1,		
	output reg	[7:0]	out2		
);
	
	reg signed [7:0] rom_1[255:0];	// Разрядность фазы 8, 64 ячейки
	
	initial 
	begin
		// Инициализируем rom_1 файлом с 32-мя отсчетами
		$readmemh("meminit/sin_full_256_8bit.mif", rom_1);
	end
	
	reg [7:0] phase_acc_1 = 0, phase_acc_2 = 0; // Регистр аккумулятора фазы, 11 бит
	wire [7:0] phase_1, phase_2;			// Переменные для фазы
	
	// Фазовый аккумулятор
	always @(posedge clk_i)
	begin
		phase_acc_1 <= phase_acc_1 +4;     // Прибавляем код DmaxL
	end
		
	assign phase_1 = phase_acc_1;		    // Оставляем начальную фазу нулевой
	assign phase_2 = phase_acc_1 + 32;  	
	 
   	always @(posedge clk_i)
	begin
		out1 <= rom_1[phase_1] + $signed({1'b1, {(7){1'b0}}});	
        out2 <= rom_1[phase_2] + $signed({1'b1, {(7){1'b0}}});
	end
		
endmodule

// Заглушка для компиляции модуля верхнего уровня
module	pll	 (	
    	input				inclk0,	
        output              c0
	);
    
endmodule