module leso7_fir(

  (* chip_pin = "23" *) 
  input          clk_50MHz,  // Вход с тактового генератора
  
  (* chip_pin = "136, 135, 133, 132, 129, 128, 127, 126" *) 
  input   [7:0]    adc_d,    // Шина данных АЦП
  
  (* chip_pin = "137" *)
  output         adc_encode, // Сигнал ENCODE АЦП
  
  (* chip_pin = "125" *)
  output        adc_pwrdwn, // Сигнал PWRDWN АЦП
  
  (* chip_pin = "106, 105, 104, 103, 100, 99, 98, 87, 86, 85, 84, 83, 80, 77" *)
  output  reg [13:0]  dac1_d,    // Шина данных ЦАП1
  
  (* chip_pin = "76" *)
  output        dac1_wrt,    // Сигнал WRT ЦАП1
  
  (* chip_pin = "75" *)
  output        dac1_clk,    // Сигнал CLK ЦАП1
  
  (* chip_pin = "72, 71, 70, 69, 68, 67, 66, 65, 60, 59, 58, 55, 54, 53" *)
  output reg  [13:0]  dac2_d,    // Шина данных ЦАП2
  
  (* chip_pin = "73" *)
  output        dac2_wrt,    // Сигнал WRT ЦАП2
  
  (* chip_pin = "74" *)
  output        dac2_clk    // Сигнал CLK ЦАП2
  // после объявления последнего порта запятая не ставится
  
);
  
  wire   signed  [7:0] signal1;   // Сигнал с АЦП  
  wire   signed  [17:0] signal2;  // Сигнал после фильтра
 
  wire clk_pll;    // объявляем линию для выхода pll
  
  pll  pll_inst (           // экземпляр модуля pll
    .inclk0 ( clk_50MHz ),  // На вход модуля pll подаем сигнал с тактового генератора
    .c0 ( clk_pll )         // Подключаем линию clk_pll к выходу pll
  );
  
  assign signal1 = $signed (adc_d -128);
  
  // Экземпляр модуля фильтра
  simple_fir 
  #(
    .N(13),   // количество коэффициентов фильтра
    .H_W(9),  // разрядность коэффициентов
    .DI_W(8), // разрядность входных данных
    .DO_W(18) // разрядность выходных данных
    ) fir_inst
  (
    .clk_i (clk_pll),
    .data_i (signal1),
    .data_o (signal2)
   ); 
  
  localparam bit_count_w = $clog2(5);
  
  always @(posedge clk_pll)
  begin
    dac1_d <= (signal1<<6) + $signed({1'b1, {(13){1'b0}}});
    dac2_d <= (signal2[17-:14]) + $signed({1'b1, {(13){1'b0}}});
  end

  assign dac1_clk = ~clk_pll;  // Подаем тактовые импульсы на вход CLK ЦАП
  assign dac1_wrt = ~clk_pll;  // Подаем тактовые ипульсы на вход WRT ЦАП
  
  assign dac2_clk = ~clk_pll;  // Подаем тактовые импульсы на вход CLK ЦАП
  assign dac2_wrt = ~clk_pll;  // Подаем тактовые ипульсы на вход WRT ЦАП
  
  assign adc_pwrdwn = 0;        // Переводим АЦП в рабочий режим
  assign adc_encode = clk_pll;  // Подаем тактовые импульсы на АЦП
      
endmodule


(* multstyle = "dsp" *) module  simple_fir
#(
  parameter     N = 5,      // количество коэффициентов 
  parameter     H_W = 9,    // разрядность h 
  parameter     DI_W = 8,   // разрядность входных данных
  parameter     DO_W = 12   // разрядность выходных данных
)
(
  input wire                    clk_i,
  input wire  signed [DI_W-1:0] data_i,
  
  // Умножаем 8-битный сигнал на 9 битные коэффициенты, получаем разрядность
  // результата 24 бит: 
  output  reg           [DO_W-1:0]    data_o 
);
// Линию задержки реализуем в виде массива регистров,
// разрядность слова DI_W, число элементов N-1 -- элементов задержки
// должно быть на единицу меньше, чем число отсчетов в h.   
  reg  signed [DI_W-1:0] r [N-2:0];   // N-1 регистров                     

  // Коэффициенты фильтра
  reg  signed [H_W-1:0] h [N-1:0];   // N регистров  

  // Результат умножения
  wire  signed [H_W + DI_W -2:0] m [N-1:0];
  
  // Прописываем значения отсчетов h
  initial begin
    // h[0] = -53;
    // h[1] = 138;
    // h[2] = 255;
    // h[3] = 138; 
    // h[4] = -53;
    h[0] = -23;
    h[1] = -51;
    h[2] = 128;
    h[3] = 68; 
    h[4] = -219;
    h[5] = -30;
    h[6] = 255;
    h[7] = -30;
    h[8] = -219; 
    h[9] = 68;
    h[10] = 128;
    h[11] = -51;
    h[12] = -23;

  end

  // Перемещаем входной сигнал по линии задержки 
  always @(posedge clk_i) begin
    r[0] <= data_i;
    r[1] <= r[0];
    r[2] <= r[1];
    r[3] <= r[2];
    r[4] <= r[3];
    r[5] <= r[4];
    r[6] <= r[5];
    r[7] <= r[6];
    r[8] <= r[7];
    r[9] <= r[8];
    r[10] <= r[9];
    r[11] <= r[10];
  end
  
  // Умножаем на отсчеты h
  assign    m[0] = data_i * h[0];
  assign    m[1] = r[0] * h[1];
  assign    m[2] = r[1] * h[2];
  assign    m[3] = r[2] * h[3];
  assign    m[4] = r[3] * h[4];
  assign    m[5] = r[4] * h[5];
  assign    m[6] = r[5] * h[6];
  assign    m[7] = r[6] * h[7];
  assign    m[8] = r[7] * h[8];
  assign    m[9] = r[8] * h[9];
  assign    m[10] = r[9] * h[10];
  assign    m[11] = r[10] * h[11];
  assign    m[12] = r[11] * h[12];

  always @(posedge clk_i) 
    data_o <= m[0] + m[1] + m[2] + m[3] + m[4] + m[5] + m[6] + m[7] + m[8] + m[9] + m[10] + m[11] + m[12];  

endmodule




(* multstyle = "dsp" *) module  fir
#(
  parameter      N = 4,
  parameter      H_W = 9   // разрядность 
)
(  
  input  wire      clk_i,
  input  wire  signed [7:0]     data_i,
  
  // Умножаем 8-битный сигнал на 16 битные коэффициенты, получаем разрядность
  // результата 24 бит: 
  output  reg           [28:0]    data_o 
);
  
  // Линию задержки реализуем в виде массива
  reg  signed [7:0] r [N-1:0];   // 7 регистров                     
  
  // Коэффициенты фильтра
  reg  signed [H_W-1:0] h [N-1:0];   // 13 регистров  
  
  reg  signed [H_W + 8:0] m [N-1:0];
  
  
  initial 
  begin
//  // с окном
//    h[0] = -236;
//    h[1] = -934;
//    h[2] = 5109;
//    h[3] = 4696;  
//    h[4] =  -21691;
//    h[5] = -3598;
//    h[6] =  32767;
//    h[7] = -3598;
//    h[8] =  -21691;
//    h[9] = 4696;
//    h[10] = 5109;
//    h[11] = -934;
//    h[12] = -236;

//    h[0] = -2956;
//    h[1] = -6601;
//    h[2] = 16482;
//    h[3] = 8697;  
//    h[4] = -28171;
//    h[5] = -3835;
//    h[6] =  32767;
//    h[7] = -3835;
//    h[8] =  -28171;
//    h[9] = 8697;
//    h[10] = 16482;
//    h[11] = -6601;
//    h[12] = -2956;
    
//    h[0] = -2;
//    h[1] = -7;
//    h[2] = 40;
//    h[3] = 37;  
//    h[4] = -169;
//    h[5] = -28;
//    h[6] =  255;
////    h[7] = -28;
////    h[8] =  -169;
////    h[9] = 37;
////    h[10] = 40;
////    h[11] = -7;
////    h[12] = -2;


    h[0] = -23;
    h[1] = -51;
    h[2] = 128;
    h[3] = 68;  
//    h[4] = -219;
//    h[5] = -30;
//    h[6] =  255;
//    h[7] = -30;
//    h[8] =  -219;
//    h[9] = 68;
//    h[10] = 128;
//    h[11] = -51;
//    h[12] = -23;

  end


  
  
//  // перемещаем входной сигнал по линии задержки 
//  always @(posedge clk_i) begin
//    
//    r[0] <= data_i;
//    r[1] <= r[0];
//    r[2] <= r[1];
//    r[3] <= r[2];
//    r[4] <= r[3];
//    r[5] <= r[4];
//    r[6] <= r[5];
//    r[7] <= r[6];
//    r[8] <= r[7];
//    r[9] <= r[8];
//    r[10] <= r[9];
//    r[11] <= r[10];
//    r[12] <= r[11];
//  end
  
    
//  always @(posedge clk_i) begin
    
//assign    m[0] = data_i * h[0];
//assign    m[1] = r[0] * h[1];
//assign    m[2] = r[1] * h[2];
//assign    m[3] = r[2] * h[3];
//assign    m[4] = r[3] * h[4];
//assign    m[5] = r[4] * h[5];
//assign    m[6] = r[5] * h[6];
//assign    m[7] = r[6] * h[7];
//assign    m[8] = r[7] * h[8];
//assign    m[9] = r[8] * h[9];
//assign    m[10] = r[9] * h[10];
//assign    m[11] = r[10] * h[11];
//assign    m[12] = r[11] * h[12];
//  end
  
  
 integer i;
 
  always@(*) begin
    for(i = 0; i < N; i = i + 1)  
      if (i == 0)
        m[i] <= data_i * h[i];
      else
        m[i] <= r[i-1] * h[i];
  end
   
   
reg           [28:0]    acc;


  always @(*) begin
    for(i = 0; i < N; i = i + 1)
      if (i == 0)
        acc = m[i];
      else
        acc = acc + m[i];
  end

  
  
  always @(posedge clk_i) begin
    for(i = 0; i < N; i = i + 1)
      if (i == 0)
        r[i] <= data_i;
      else
        r[i] <= r[i-1];
  end
  
  



  

always @(posedge clk_i) 
    data_o = acc; 
    
    
 // data_o = m[0] + m[1] + m[2] + m[3] + m[4] + m[5] + m[6] + m[7] + m[8] + m[9] + m[10] + m[11] + m[12];  
  
              
     

endmodule

