module leso7_fir(

  (* chip_pin = "23" *) 
  input          clk_50MHz,  // Вход с тактового генератора
  
  (* chip_pin = "136, 135, 133, 132, 129, 128, 127, 126" *) 
  input   [7:0]    adc_d,    // Шина данных АЦП
  
  (* chip_pin = "137" *)
  output         adc_encode, // Сигнал ENCODE АЦП
  
  (* chip_pin = "125" *)
  output        adc_pwrdwn, // Сигнал PWRDWN АЦП
  
  (* chip_pin = "106, 105, 104, 103, 100, 99, 98, 87, 86, 85, 84, 83, 80, 77" *)
  output  reg [13:0]  dac1_d,    // Шина данных ЦАП1
  
  (* chip_pin = "76" *)
  output        dac1_wrt,    // Сигнал WRT ЦАП1
  
  (* chip_pin = "75" *)
  output        dac1_clk,    // Сигнал CLK ЦАП1
  
  (* chip_pin = "72, 71, 70, 69, 68, 67, 66, 65, 60, 59, 58, 55, 54, 53" *)
  output reg  [13:0]  dac2_d,    // Шина данных ЦАП2
  
  (* chip_pin = "73" *)
  output        dac2_wrt,    // Сигнал WRT ЦАП2
  
  (* chip_pin = "74" *)
  output        dac2_clk    // Сигнал CLK ЦАП2
  // после объявления последнего порта запятая не ставится
  
);
  
  wire   signed  [7:0] signal1;   // Сигнал с АЦП  
  wire   signed  [17:0] signal2;  // Сигнал после фильтра
 
  wire clk_pll;    // объявляем линию для выхода pll
  
  pll  pll_inst (           // экземпляр модуля pll
    .inclk0 ( clk_50MHz ),  // На вход модуля pll подаем сигнал с тактового генератора
    .c0 ( clk_pll )         // Подключаем линию clk_pll к выходу pll
  );
  
  assign signal1 = $signed (adc_d -128);
  
  // Экземпляр модуля фильтра
  fir_sym 
  #(
    .N(13),    // количество коэффициентов фильтра
    .H_W(9),  // разрядность коэффициентов
    .DI_W(8), // разрядность входных данных
    .DO_W(18) // разрядность выходных данных
    ) fir_inst
  (
    .clk_i (clk_pll),
    .data_i (signal1),
    .data_o (signal2)
   ); 
  
  
  always @(posedge clk_pll)
  begin
    dac1_d <= (signal1 << 6) + $signed({1'b1, {(13){1'b0}}});
    dac2_d <= (signal2[17-:14] << 0) + $signed({1'b1, {(13){1'b0}}});
  end

  assign dac1_clk = ~clk_pll;  // Подаем тактовые импульсы на вход CLK ЦАП
  assign dac1_wrt = ~clk_pll;  // Подаем тактовые ипульсы на вход WRT ЦАП
  
  assign dac2_clk = ~clk_pll;  // Подаем тактовые импульсы на вход CLK ЦАП
  assign dac2_wrt = ~clk_pll;  // Подаем тактовые ипульсы на вход WRT ЦАП
  
  assign adc_pwrdwn = 0;        // Переводим АЦП в рабочий режим
  assign adc_encode = clk_pll;  // Подаем тактовые импульсы на АЦП
      
endmodule


(* multstyle = "dsp" *) module  simple_fir
#(
  parameter     N = 5,      // количество коэффициентов 
  parameter     H_W = 9,    // разрядность h 
  parameter     DI_W = 8,   // разрядность входных данных
  parameter     DO_W = 18   // разрядность выходных данных
)
(
  input wire                    clk_i,
  input wire  signed [DI_W-1:0] data_i,
  output  reg        [DO_W-1:0] data_o 
);
// Линию задержки реализуем в виде массива регистров,
// разрядность слова DI_W, число элементов N-1 -- элементов задержки
// должно быть на единицу меньше, чем число отсчетов в h.   
  reg  signed [DI_W-1:0] r [N-2:0];   // N-1 регистров                     

  // Коэффициенты фильтра
  reg  signed [H_W-1:0] h [N-1:0];   // N регистров  

  // Результат умножения
  wire  signed [H_W + DI_W -1:0] m [N-1:0];
  
  // Прописываем значения отсчетов h
  initial begin
    h[0] = -53;
    h[1] = 138;
    h[2] = 255;
    h[3] = 138; 
    h[4] = -53;
  end

  // Перемещаем входной сигнал по линии задержки 
  always @(posedge clk_i) begin
    r[0] <= data_i;
    r[1] <= r[0];
    r[2] <= r[1];
    r[3] <= r[2];
  end
  
  // Умножаем на отсчеты h
  assign    m[0] = data_i * h[0];
  assign    m[1] = r[0] * h[1];
  assign    m[2] = r[1] * h[2];
  assign    m[3] = r[2] * h[3];
  assign    m[4] = r[3] * h[4];

  always @(posedge clk_i) 
    data_o <= m[0] + m[1] + m[2] + m[3] + m[4];  

endmodule




(* multstyle = "dsp" *) module  fir
#(
  parameter     N = 5,      // количество коэффициентов 
  parameter     H_W = 9,    // разрядность h 
  parameter     DI_W = 8,   // разрядность входных данных
  parameter     DO_W = 18   // разрядность выходных данных
)
(  
  input wire                    clk_i,
  input wire  signed [DI_W-1:0] data_i,
  output  reg        [DO_W-1:0] data_o 
);
// Линию задержки реализуем в виде массива регистров,
// разрядность слова DI_W, число элементов N-1 -- элементов задержки
// должно быть на единицу меньше, чем число отсчетов в h.   
  reg  signed [DI_W-1:0] r [N-2:0];   // N-1 регистров                     

  // Коэффициенты фильтра
  reg  signed [H_W-1:0] h [N-1:0];   // N регистров  

  // Результат умножения
  reg  signed [H_W + DI_W -1:0] m [N-1:0];
   
  initial begin
    // h[0] = -53;
    // h[1] = 138;
    // h[2] = 255;
    // h[3] = 138; 
    // h[4] = -53;
    // // // Полосовой
    // h[0] = -23;
    // h[1] = -51;
    // h[2] = 128;
    // h[3] = 68; 
    // h[4] = -219;
    // h[5] = -30;
    // h[6] = 255;
    // h[7] = -30;
    // h[8] = -219; 
    // h[9] = 68;
    // h[10] = 128;
    // h[11] = -51;
    // h[12] = -23;
    
    // // // Полосовой W
    // h[0] = -2;
    // h[1] = -7;
    // h[2] = 40;
    // h[3] = 37; 
    // h[4] = -169;
    // h[5] = -28;
    // h[6] = 255;
    // h[7] = -28;
    // h[8] = -169; 
    // h[9] = 37;
    // h[10] = 40;
    // h[11] = -7;
    // h[12] = -2;
    
    // // ФНЧ N=13, 8кгЦ, 15кГц
    // h[0] = -80;
    // h[1] = -42;
    // h[2] = 25;
    // h[3] = 105; 
    // h[4] = 182;
    // h[5] = 236;
    // h[6] = 255;
    // h[7] = 236;
    // h[8] = 182; 
    // h[9] = 105;
    // h[10] = 25;
    // h[11] = -42;
    // h[12] = -80;
    
    // ФНЧ: N=13, 16кгЦ, 23кГц.
    h[0] = 42;
    h[1] = -13;
    h[2] = -62;
    h[3] = -36; 
    h[4] = 73;
    h[5] = 199;
    h[6] = 255;
    h[7] = 199;
    h[8] = 73; 
    h[9] = -36;
    h[10] = -62;
    h[11] = -13;
    h[12] = 42;
    // h[12] = -74;
    
    
    
    // h[0] = -80;
    // h[1] = -42;
    // h[2] = 25;
    // h[3] = 105; 
    // h[4] = 182;
    // h[5] = 236;
    // h[6] = 255;
    // h[7] = 236;
    // h[8] = 182; 
    // h[9] = 105;
    // h[10] = 25;
    // h[11] = -42;
    // h[12] = -80;
    
    // h[0] = -74;
    // h[1] = -39;
    // h[2] = 23;
    // h[3] = 98; 
    // h[4] = 168;
    // h[5] = 218;
    // h[6] = 236;
    // h[7] = 218;
    // h[8] = 168; 
    // h[9] = 98;
    // h[10] = 23;
    // h[11] = -39;
    // h[12] = -74;
    
  end

  integer i;
  reg  signed    [DO_W-1:0]    acc;

  always @(posedge clk_i) begin
    for(i = 0; i < N-1; i = i + 1)
      if (i == 0)
        r[i] <= data_i;
      else
        r[i] <= r[i-1];
  end
  
  always @(*) begin
    for(i = 0; i < N; i = i + 1)  
      if (i == 0)
        m[i] <= data_i * h[i];
      else
        m[i] <= r[i-1] * h[i];
  end
 
  always @(*) begin
    for(i = 0; i < N; i = i + 1)
      if (i == 0)
        acc = m[i];
      else
        acc = acc + m[i];
  end
  

always @(posedge clk_i) 
    data_o = acc; 
    
endmodule

(* multstyle = "dsp" *) module  fir_sym
#(
  parameter     N = 5,      // количество коэффициентов 
  parameter     H_W = 9,    // разрядность h 
  parameter     DI_W = 8,   // разрядность входных данных
  parameter     DO_W = 18   // разрядность выходных данных
)
(  
  input wire                    clk_i,
  input wire  signed [DI_W-1:0] data_i,
  output  reg        [DO_W-1:0] data_o 
);
// Линию задержки реализуем в виде массива регистров,
// разрядность слова DI_W, число элементов N-1 -- элементов задержки
// должно быть на единицу меньше, чем число отсчетов в h.   
  reg  signed [DI_W-1:0] r [N-2:0];   // N-1 регистров                     

  // Коэффициенты фильтра
  reg  signed [H_W-1:0] h [N/2 :0];   // N регистров  

 
  // Результат умножения
  reg  signed [H_W + DI_W :0] m [N/2 :0];
   
  initial begin
    // // ФНЧ: N=13, 16кгЦ, 23кГц.
    // h[0] = 42;
    // h[1] = -13;
    // h[2] = -62;
    // h[3] = -36; 
    // h[4] = 73;
    // h[5] = 199;
    // h[6] = 255;
    
    // // ФНЧ: N=13, 23.1кгЦ, 30.7кГц.
    // h[0] = -24;
    // h[1] = 32;
    // h[2] = 21;
    // h[3] = -51; 
    // h[4] = -19;
    // h[5] = 151;
    // h[6] = 255;
    
    // // ФНЧ: N=13, 38.5кгЦ, 46кГц.
    // h[0] = -6;
    // h[1] = 16;
    // h[2] = -26;
    // h[3] = 35; 
    // h[4] = -41;
    // h[5] = 45;
    // h[6] = 255;
    
    // // ФНЧ: N=17, 35.3кгЦ, 41кГц.
    
    // h[0] = 7;
    // h[1] = -18;
    // h[2] = 21;
    // h[3] = -13; 
    // h[4] = -5;
    // h[5] = 30;
    // h[6] = -54;
    // h[7] = 72;
    // h[8] = 255;
    
//    // ФНЧ: N=17, 6кГц, 11,7кГц..
//    h[0] = -82;
//    h[1] = -60;
//    h[2] = -17;
//    h[3] = 38; 
//    h[4] = 101;
//    h[5] = 161;
//    h[6] = 211;
//    h[7] = 244;
//    h[8] = 255;
    
    // // ФНЧ: N=13, 16кГц, 23кГц..
    // h[0] = 21;
    // h[1] = -7;
    // h[2] = -31;
    // h[3] = -18; 
    // h[4] = 36;
    // h[5] = 99;
    // h[6] = 127;
    
    
    // ФНЧ: N=13, 16кГц, 23кГц Window
    h[0] = 3;
    h[1] = -2;
    h[2] = -19;
    h[3] = -19; 
    h[4] = 56;
    h[5] = 187;
    h[6] = 255;

    
    
  end

  integer i;
  reg  signed    [DO_W-1:0]    acc;

  always @(posedge clk_i) begin
    for(i = 0; i < N-1; i = i + 1)
      if (i == 0)
        r[i] <= data_i;
      else
        r[i] <= r[i-1];
  end

    
  always @(*) begin
    for(i = 0; i < N/2 +1; i = i + 1)  
      if (i == 0)
        m[i] <= (data_i + r[N -2 -i]) * h[i];
      else if (i == N/2)
        m[i] <= r[N -2 -i] * h[i];
      else
        m[i] <= (r[i-1] + r[N -2 -i]) * h[i];
  end
 
  always @(*) begin
    for(i = 0; i < N/2 +1; i = i + 1)
      if (i == 0)
        acc = m[i];
      else
        acc = acc + m[i];
  end
  

always @(posedge clk_i) 
    data_o = acc; 
    
endmodule
